@@ -273,17 +273,24 @@ impl slumber_template::Context for SingleRenderContext<'_> {
             "env" => functions::env(arguments),
             "file" => functions::file(arguments),
             "float" => functions::float(arguments),
+            "index" => functions::index(arguments),
             "integer" => functions::integer(arguments),
+            "join" => functions::join(arguments),
             "jq" => functions::jq(arguments),
             "json_parse" => functions::json_parse(arguments),
             "jsonpath" => functions::jsonpath(arguments),
+            "lower" => functions::lower(arguments),
             "prompt" => functions::prompt(arguments).await,
+            "replace" => functions::replace(arguments),
             "response" => functions::response(arguments).await,
             "response_header" => functions::response_header(arguments).await,
             "select" => functions::select(arguments).await,
             "sensitive" => functions::sensitive(arguments),
+            "slice" => functions::slice(arguments),
+            "split" => functions::split(arguments),
             "string" => functions::string(arguments),
             "trim" => functions::trim(arguments),
+            "upper" => functions::upper(arguments),
             _ => Err(RenderError::FunctionUnknown),
         }
     }
@@ -534,6 +541,12 @@ pub enum FunctionError {
     #[error("Unknown recipe `{recipe_id}`")]
     RecipeUnknown { recipe_id: RecipeId },
 
+    /// Invalid regular expression given
+    ///
+    /// [regex::Error] is pretty descriptive so we don't need extra context.
+    #[error(transparent)]
+    Regex(#[from] regex::Error),
+
     /// Recipe for `response()`/`response_header()` has no history
     #[error("No response available")]
     ResponseMissing,
