@@ -118,8 +118,8 @@ async fn test_base64(
 #[case::string_zero("0".into(), true)]
 #[case::bytes_empty(b"".into(), false)]
 #[case::bytes_invalid(invalid_utf8().into(), true)]
-#[case::array_empty(vec![].into(), false)]
-#[case::array_filled(vec!["1".into(), "2".into()].into(), true)]
+#[case::array_empty((vec![] as Vec<i64>).into(), false)]
+#[case::array_filled(vec!["1", "2"].into(), true)]
 #[tokio::test]
 async fn test_boolean(#[case] input: Expression, #[case] expected: bool) {
     let template = Template::function_call("boolean", [input], []);
@@ -215,11 +215,8 @@ async fn test_command(
 /// This ensures we don't execute the command for previews
 #[tokio::test]
 async fn test_command_lazy() {
-    let template = Template::function_call(
-        "command",
-        [vec!["i-will-fail".into()].into()],
-        [],
-    );
+    let template =
+        Template::function_call("command", [vec!["i-will-fail"].into()], []);
     // This shouldn't fail because the command isn't evaluated yet
     let output = template
         .render(&TemplateContext::factory(()).streaming(true))
@@ -267,14 +264,20 @@ async fn test_concat(
 
 /// `env()`
 #[rstest]
-#[case::set("CARGO_PKG_NAME", Ok("slumber_core"))]
-#[case::unset("NOT_A_REAL_ENV_VAR", Ok(""))]
+#[case::set("CARGO_PKG_NAME", None, Ok("slumber_core"))]
+#[case::unset("NOT_A_REAL_ENV_VAR", None, Ok(""))]
+#[case::unset_default("NOT_A_REAL_ENV_VAR", Some("default"), Ok("default"))]
 #[tokio::test]
 async fn test_env(
     #[case] variable: &str,
+    #[case] default: Option<&str>,
     #[case] expected: Result<&str, &str>,
 ) {
-    let template = Template::function_call("env", [variable.into()], []);
+    let template = Template::function_call(
+        "env",
+        [variable.into()],
+        [("default", default.map(Expression::from))],
+    );
     assert_result(
         template
             .render_bytes(&TemplateContext::factory(()).streaming(false))
@@ -358,7 +361,7 @@ async fn test_file_tilde(temp_dir: TempDir) {
 #[case::bool_false(false.into(), Ok(0.0))]
 #[case::bool_true(true.into(), Ok(1.0))]
 #[case::array(
-    vec!["1".into(), "2".into()].into(),
+    vec!["1", "2"].into(),
     Err("Expected one of float, integer, boolean, or string/bytes"),
 )]
 #[tokio::test]
@@ -377,6 +380,35 @@ async fn test_float(
     );
 }
 
+/// `index()`
+#[rstest]
+#[case::empty_string("".into(), 0, None)]
+#[case::empty_array(Vec::<&str>::new().into(), 0, None)]
+#[case::empty_negative("".into(), -1, None)] // Check for div by zero
+#[case::string("abc".into(), 1, Some("b".into()))]
+#[case::array(vec!["a", "b", "c"].into(), 1, Some("b".into()))]
+#[case::negative("abc".into(), -2, Some("b".into()))]
+#[case::utf8_at("nägemist".into(), 1, Some("ä".into()))]
+#[case::utf8_after("nägemist".into(), 2, Some("g".into()))]
+#[case::utf8_bytes(b"n\xc3\xa4gemist".into(), 2, Some(b"\xa4".into()))]
+#[tokio::test]
+async fn test_index(
+    #[case] value: Expression,
+    #[case] index: i64,
+    #[case] expected: Option<Value>,
+) {
+    let template = Template::function_call("index", [index.into(), value], []);
+    assert_eq!(
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await
+            .unwrap(),
+        expected.into(),
+    );
+}
+
 /// `integer()`
 #[rstest]
 #[case::null(Expression::Literal(Literal::Null), Ok(0))]
@@ -390,7 +422,7 @@ async fn test_float(
 #[case::bool_false(false.into(), Ok(0))]
 #[case::bool_true(true.into(), Ok(1))]
 #[case::array(
-    vec!["1".into(), "2".into()].into(),
+    vec!["1", "2"].into(),
     Err("Expected one of integer, float, boolean, or string/bytes"),
 )]
 #[tokio::test]
@@ -409,6 +441,32 @@ async fn test_integer(
     );
 }
 
+/// `join()`
+#[rstest]
+// These are the inverse of the split() test cases
+#[case::empty_array(vec![] as Vec<&str>, ",", "")]
+#[case::empty_separator(vec!["", "a", "b", "c", ""], "", "abc")]
+#[case::empty_empty(vec!["", ""], "", "")]
+#[case::comma(vec!["a", "b", "c", ""], ",", "a,b,c,")]
+#[case::numbers(vec![1, 2, 3], ",", "1,2,3")]
+#[tokio::test]
+async fn test_join(
+    #[case] values: Vec<impl Into<Expression>>,
+    #[case] separator: &str,
+    #[case] expected: &str,
+) {
+    let values = values.into_iter().map(Into::into).collect();
+    let template =
+        Template::function_call("join", [separator.into(), values], []);
+    assert_eq!(
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await
+            .unwrap(),
+        expected,
+    );
+}
+
 /// `jq()`
 #[rstest]
 // Default mode is auto
@@ -569,6 +627,22 @@ async fn test_jsonpath(
     );
 }
 
+/// `lower()`
+#[rstest]
+#[case::ascii("heLLo", "hello")]
+#[case::utf8("NÄGEMIST", "nägemist")]
+#[tokio::test]
+async fn test_lower(#[case] input: &str, #[case] expected: &str) {
+    let template = Template::function_call("lower", [input.into()], []);
+    assert_eq!(
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await
+            .unwrap(),
+        expected
+    );
+}
+
 /// `prompt()`
 #[rstest]
 #[case::reply(Some("test"), None, false, Ok("test"))]
@@ -604,6 +678,53 @@ async fn test_prompt(
     );
 }
 
+/// `replace()`
+#[rstest]
+#[case::non_overlapping("banana", "na", "ma", false, None, Ok("bamama"))]
+#[case::overlapping("banana", "ana", "ama", false, None, Ok("bamana"))]
+#[case::regex("banana", "[ab]", "x", true, None, Ok("xxnxnx"))]
+#[case::regex_n("banana", "[ab]", "x", true, Some(2), Ok("xxnana"))]
+#[case::regex_overlapping("banana", "a.a", "axa", true, Some(3), Ok("baxana"))]
+#[case::n_limited("banana", "na", "ma", false, Some(1), Ok("bamana"))]
+#[case::n_zero("banana", "na", "ma", false, Some(0), Ok("banana"))]
+#[case::n_over("banana", "na", "ma", false, Some(7), Ok("bamama"))]
+#[case::n_over("banana", "na", "ma", false, Some(7), Ok("bamama"))]
+#[case::error_n_invalid(
+    "", "", "", false, Some(-1), Err("Integer out of range [0, 4294967295]"
+))]
+#[case::error_regex_invalid(
+    "banana",
+    "[a",
+    "",
+    true,
+    None,
+    Err("regex parse error")
+)]
+#[tokio::test]
+async fn test_replace(
+    #[case] value: &str,
+    #[case] from: &str,
+    #[case] to: &str,
+    #[case] regex: bool,
+    #[case] n: Option<i64>,
+    #[case] expected: Result<&str, &str>,
+) {
+    let template = Template::function_call(
+        "replace",
+        [from.into(), to.into(), value.into()],
+        [
+            ("regex", Some(regex.into())),
+            ("n", n.map(Expression::from)),
+        ],
+    );
+    assert_result(
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await,
+        expected,
+    );
+}
+
 /// `response()`
 #[rstest]
 // ===== Response is cached =====
@@ -900,12 +1021,90 @@ async fn test_sensitive(#[case] input: &str, #[case] expected: &str) {
     );
 }
 
+/// `slice()`
+#[rstest]
+#[case::empty("".into(), 0, Some(0), "".into())]
+#[case::empty_negative("".into(), -1, None, "".into())] // Check for div by zero
+#[case::string_zeroes("abc".into(), 0, Some(0), "".into())]
+#[case::string_len("abc".into(), 3, Some(3), "".into())]
+#[case::string_partial("abc".into(), 1, Some(3), "bc".into())]
+#[case::string_full("abc".into(), 0, Some(3), "abc".into())]
+#[case::array_zeroes(vec![0, 1, 2].into(), 0, Some(0), Vec::<i64>::new().into())]
+#[case::array_len(vec![0, 1, 2].into(), 3, Some(3), Vec::<i64>::new().into())]
+#[case::array_partial(vec![0, 1, 2].into(), 1, Some(3), vec![1, 2].into())]
+#[case::array_full(vec![0, 1, 2].into(), 0, Some(3), vec![0, 1, 2].into())]
+#[case::stop_less_than_start("abc".into(), 2, Some(1), "".into())]
+#[case::start_negative("abc".into(), -2, Some(3), "bc".into())]
+#[case::stop_negative("abc".into(), 1, Some(-1), "b".into())]
+#[case::stop_negative_wrap("abc".into(), 1, Some(-4), "b".into())] // -4 => -1
+#[case::stop_null("abc".into(), 1, None, "bc".into())]
+#[case::stop_high("abc".into(), 1, Some(5), "bc".into())]
+#[case::utf8_string("nägemist".into(), 1, Some(3), "äg".into())]
+#[case::utf8_bytes_partial_char(b"n\xc3\xa4gemist".into(), 1, Some(2), b"\xc3".into())]
+#[case::utf8_bytes_full_char(b"n\xc3\xa4gemist".into(), 1, Some(3), "ä".into())]
+#[tokio::test]
+async fn test_slice(
+    #[case] value: Expression,
+    #[case] start: i64,
+    #[case] stop: Option<i64>,
+    #[case] expected: Value,
+) {
+    let template = Template::function_call(
+        "slice",
+        [start.into(), stop.into(), value],
+        [],
+    );
+    assert_eq!(
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await
+            .unwrap(),
+        expected,
+    );
+}
+
+/// `split()`
+#[rstest]
+#[case::empty_string("", ",", None, Ok(vec![""]))]
+#[case::empty_separator("abc", "", None, Ok(vec!["", "a", "b", "c", ""]))]
+#[case::empty_empty("", "", None, Ok(vec!["", ""]))]
+#[case::comma("a,b,c,", ",", None, Ok(vec!["a", "b", "c", ""]))]
+#[case::n_limited("a,b,c,", ",", Some(2), Ok(vec!["a", "b", "c,"]))]
+#[case::n_zero("a,b,c,", ",", Some(0), Ok(vec!["a,b,c,"]))]
+#[case::n_over("a,b,c,", ",", Some(7), Ok(vec!["a", "b", "c", ""]))]
+#[case::error_n_invalid(
+    "", ",", Some(-1), Err("argument n=-1: Integer out of range [0, 4294967295]"
+))]
+#[tokio::test]
+async fn test_split(
+    #[case] value: &str,
+    #[case] separator: &str,
+    #[case] n: Option<i64>,
+    #[case] expected: Result<Vec<&str>, &str>,
+) {
+    let template = Template::function_call(
+        "split",
+        [separator.into(), value.into()],
+        [("n", n.map(Expression::from))],
+    );
+    assert_result(
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await,
+        expected.map(Value::from),
+    );
+}
+
 /// `string()`
 #[rstest]
 #[case::primitive(true.into(), Ok("true"))]
 #[case::string("test".into(), Ok("test"))]
 #[case::bytes(b"test".into(), Ok("test"))]
-#[case::array(vec!["a".into(), "b".into()].into(), Ok("['a', 'b']"))]
+#[case::array(vec!["a", "b"].into(), Ok("['a', 'b']"))]
 #[case::error_invalid_utf8(invalid_utf8().into(), Err("invalid utf-8"))]
 #[tokio::test]
 async fn test_string(
@@ -947,6 +1146,22 @@ async fn test_trim(
     );
 }
 
+/// `upper()`
+#[rstest]
+#[case::ascii("heLLo", "HELLO")]
+#[case::utf8("nägemist", "NÄGEMIST")]
+#[tokio::test]
+async fn test_upper(#[case] input: &str, #[case] expected: &str) {
+    let template = Template::function_call("upper", [input.into()], []);
+    assert_eq!(
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await
+            .unwrap(),
+        expected
+    );
+}
+
 /// Test that the stream source is retained for a single-chunk template
 #[rstest]
 #[case::stream_root("{{ file('data.json') }}", true)]
