@@ -131,6 +131,15 @@ impl<const N: usize> From<&'static [u8; N]> for Value {
     }
 }
 
+impl<T> From<Option<T>> for Value
+where
+    Value: From<T>,
+{
+    fn from(value: Option<T>) -> Self {
+        value.map(Value::from).unwrap_or(Value::Null)
+    }
+}
+
 impl<T> From<Vec<T>> for Value
 where
     Value: From<T>,
@@ -271,6 +280,27 @@ impl TryFromValue for i64 {
     }
 }
 
+impl TryFromValue for u32 {
+    fn try_from_value(value: Value) -> Result<Self, WithValue<ValueError>> {
+        match &value {
+            Value::Integer(i) => (*i).try_into().map_err(|_| {
+                WithValue::new(
+                    value,
+                    ValueError::IntegerRange {
+                        expected: format!("[{}, {}]", u32::MIN, u32::MAX),
+                    },
+                )
+            }),
+            _ => Err(WithValue::new(
+                value,
+                ValueError::Type {
+                    expected: Expected::Integer,
+                },
+            )),
+        }
+    }
+}
+
 impl TryFromValue for String {
     fn try_from_value(value: Value) -> Result<Self, WithValue<ValueError>> {
         // This will succeed for anything other than invalid UTF-8 bytes
@@ -495,12 +525,6 @@ where
     }
 }
 
-impl<T: FunctionOutput> FunctionOutput for Option<T> {
-    fn into_result(self) -> Result<LazyValue, RenderError> {
-        self.map(T::into_result).unwrap_or(Ok(Value::Null.into()))
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
