@@ -19,7 +19,7 @@ use slumber_core::{
     render::{Prompter, TemplateContext},
 };
 use slumber_template::{RenderedOutput, Template};
-use slumber_util::ResultTraced;
+use slumber_util::{ResultTraced, yaml::SourceLocation};
 use std::{
     path::{Path, PathBuf},
     sync::Arc,
@@ -69,7 +69,7 @@ impl TuiState {
                     Self(TuiStateInner::Error {
                         database,
                         collection_file,
-                        error,
+                        error: error.into(),
                         messages_tx,
                         _watcher: watcher,
                     })
@@ -297,7 +297,9 @@ impl LoadedState {
                         Ok(collection) => {
                             Message::CollectionEndReload(collection)
                         }
-                        Err(error) => Message::Error { error },
+                        Err(error) => Message::Error {
+                            error: error.into(),
+                        },
                     };
                     messages_tx.send(message);
                 });
@@ -305,10 +307,8 @@ impl LoadedState {
             Message::CollectionEndReload(collection) => {
                 self.reload_collection(collection);
             }
-            Message::CollectionEdit => {
-                let path = self.collection_file.path().to_owned();
-                let command = TuiContext::get().config.editor_command(&path)?;
-                util::yield_terminal(command, &self.messages_tx)?;
+            Message::CollectionEdit { location } => {
+                self.edit_collection(location)?;
             }
             Message::CopyRequestUrl => self.copy_request_url()?,
             Message::CopyRequestBody => self.copy_request_body()?,
@@ -325,16 +325,20 @@ impl LoadedState {
                 )?;
             }
             Message::FileEdit { file, on_complete } => {
-                let command =
-                    TuiContext::get().config.editor_command(file.path())?;
-                util::yield_terminal(command, &self.messages_tx)?;
+                let editor = TuiContext::get().config.editor()?;
+                util::yield_terminal(
+                    editor.open(file.path()),
+                    &self.messages_tx,
+                )?;
                 on_complete(file);
             }
             Message::FileView { file, mime } => {
-                let command = TuiContext::get()
-                    .config
-                    .pager_command(file.path(), mime.as_ref())?;
-                util::yield_terminal(command, &self.messages_tx)?;
+                let pager =
+                    TuiContext::get().config.tui.pager(mime.as_ref())?;
+                util::yield_terminal(
+                    pager.open(file.path()),
+                    &self.messages_tx,
+                )?;
                 // Dropping the file deletes it, so we can't do it until after
                 // the command is done
                 drop(file);
@@ -434,6 +438,20 @@ impl LoadedState {
         self.view.notify("Reloaded collection");
     }
 
+    /// Open the collection file in the user's editor
+    fn edit_collection(
+        &self,
+        location: Option<SourceLocation>,
+    ) -> anyhow::Result<()> {
+        let editor = TuiContext::get().config.editor()?;
+        let command = if let Some(location) = location {
+            editor.open_at(location.source, location.line, location.column)
+        } else {
+            editor.open(self.collection_file.path())
+        };
+        util::yield_terminal(command, &self.messages_tx)
+    }
+
     /// Update the collection name in the DB according to the loaded collection.
     /// Call this whenever the collection is successfully loaded to ensure the
     /// DB is kept up to date.
@@ -623,7 +641,7 @@ impl LoadedState {
         match result {
             Ok(exchange) => {
                 // Persist in the DB if not disabled by global config or recipe
-                let persist = TuiContext::get().config.persist
+                let persist = TuiContext::get().config.tui.persist
                     && self
                         .collection
                         .recipes
