@@ -7,19 +7,19 @@ use crate::{
     view::ViewContext,
 };
 use ratatui::{
+    Frame, Terminal,
     backend::TestBackend,
     layout::{Position, Rect},
     text::Line,
-    Frame, Terminal,
 };
 use rstest::fixture;
-use slumber_core::{
-    collection::Collection, db::CollectionDatabase, test_util::Factory,
-};
-use std::{cell::RefCell, future::Future, rc::Rc, sync::Arc};
+use slumber_core::{collection::Collection, database::CollectionDatabase};
+use slumber_util::Factory;
+use std::{cell::RefCell, future::Future, rc::Rc, sync::Arc, time::Duration};
 use tokio::{
     sync::mpsc::{self, UnboundedReceiver},
     task::LocalSet,
+    time,
 };
 
 /// Get a test harness, with a clean terminal etc. See [TestHarness].
@@ -76,9 +76,14 @@ impl TestHarness {
         self.messages_rx.try_recv().expect("Message queue empty")
     }
 
-    /// Pop the next message off the queue, waiting if empty
+    /// Pop the next message off the queue, waiting if empty. This will wait
+    /// with a 1s timeout to prevent missing messages from blocking a test
+    /// forever.
     pub async fn pop_message_wait(&mut self) -> Message {
-        self.messages_rx.recv().await.expect("Message queue closed")
+        time::timeout(Duration::from_secs(1), self.messages_rx.recv())
+            .await
+            .expect("Message did not appear within 1s timeout")
+            .expect("Message queue closed")
     }
 
     /// Clear all messages in the queue
@@ -134,7 +139,7 @@ impl TestTerminal {
         &self,
         expected: impl IntoIterator<Item = impl Into<Line<'a>>>,
     ) {
-        self.0.borrow().backend().assert_buffer_lines(expected)
+        self.0.borrow().backend().assert_buffer_lines(expected);
     }
 
     pub fn draw(&self, f: impl FnOnce(&mut Frame)) {
@@ -157,10 +162,14 @@ pub async fn run_local<T>(future: impl Future<Output = T>) -> T {
 macro_rules! assert_events {
     ($($pattern:pat $(if $condition:expr)?),* $(,)?) => {
         ViewContext::inspect_event_queue(|events| {
-            // In order to support conditions on each individual event, we have
-            // to unpack them here
+            // Can't use expect(unused_mut) because not all invocations trigger
+            // the warning
+            #[allow(clippy::allow_attributes)]
             #[allow(unused_mut)]
             let mut len = 0;
+
+            // In order to support conditions on each individual event, we have
+            // to unpack them here
             $(
                 let Some(event) = events.get(len) else {
                     panic!(
@@ -168,7 +177,7 @@ macro_rules! assert_events {
                         expected = stringify!($pattern),
                     );
                 };
-                slumber_core::assert_matches!(event, $pattern $(if $condition)?);
+                slumber_util::assert_matches!(event, $pattern $(if $condition)?);
                 len += 1;
             )*
             // Make sure there aren't any trailing events
