@@ -1,17 +1,9 @@
 use crate::{
     context::TuiContext,
-    util::ResultReported,
     view::{
-        ViewContext,
-        common::{
-            actions::{IntoMenuAction, MenuAction},
-            modal::Modal,
-            table::Table,
-            text_box::TextBox,
-        },
+        common::{actions::MenuAction, table::Table},
         component::{
             Component,
-            misc::TextBoxModal,
             recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
         },
         context::UpdateContext,
@@ -27,12 +19,12 @@ use ratatui::{
 use slumber_config::Action;
 use slumber_core::collection::{Authentication, RecipeId};
 use slumber_template::Template;
-use strum::{EnumCount, EnumIter, IntoEnumIterator};
+use strum::{EnumCount, EnumIter};
 
 /// Display authentication settings for a recipe
 #[derive(Debug)]
 pub struct AuthenticationDisplay {
-    /// Emitter for the callback from editing the body
+    /// Emitter for the callback from editing the authentication field(s)
     override_emitter: Emitter<SaveAuthenticationOverride>,
     /// Emitter for menu actions
     actions_emitter: Emitter<AuthenticationMenuAction>,
@@ -72,8 +64,8 @@ impl AuthenticationDisplay {
             },
         };
         Self {
-            override_emitter: Default::default(),
-            actions_emitter: Default::default(),
+            override_emitter: Emitter::default(),
+            actions_emitter: Emitter::default(),
             state,
         }
     }
@@ -113,8 +105,8 @@ impl EventHandler for AuthenticationDisplay {
             })
             .emitted(
                 self.override_emitter,
-                |SaveAuthenticationOverride(value)| {
-                    self.state.set_override(&value);
+                |SaveAuthenticationOverride(template)| {
+                    self.state.set_override(template);
                 },
             )
             .emitted(self.actions_emitter, |menu_action| match menu_action {
@@ -126,9 +118,16 @@ impl EventHandler for AuthenticationDisplay {
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        AuthenticationMenuAction::iter()
-            .map(MenuAction::with_data(self, self.actions_emitter))
-            .collect()
+        let emitter = self.actions_emitter;
+        vec![
+            emitter
+                .menu(AuthenticationMenuAction::Edit, "Edit Authentication")
+                .shortcut(Some(Action::Edit)),
+            emitter
+                .menu(AuthenticationMenuAction::Reset, "Reset Authentication")
+                .enable(self.state.is_overridden())
+                .shortcut(Some(Action::Reset)),
+        ]
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -189,35 +188,16 @@ impl Draw for AuthenticationDisplay {
 }
 
 /// Local event to save a user's override value(s). Triggered from the edit
-/// modal. These will be raw string values, consumer has to parse them to
-/// templates.
+/// modal.
 #[derive(Debug)]
-struct SaveAuthenticationOverride(String);
+struct SaveAuthenticationOverride(Template);
 
-#[derive(Copy, Clone, Debug, derive_more::Display, EnumIter)]
+#[derive(Copy, Clone, Debug)]
 enum AuthenticationMenuAction {
-    #[display("Edit Authentication")]
     Edit,
-    #[display("Reset Authentication")]
     Reset,
 }
 
-impl IntoMenuAction<AuthenticationDisplay> for AuthenticationMenuAction {
-    fn enabled(&self, data: &AuthenticationDisplay) -> bool {
-        match self {
-            Self::Edit => true,
-            Self::Reset => data.state.is_overridden(),
-        }
-    }
-
-    fn shortcut(&self, _: &AuthenticationDisplay) -> Option<Action> {
-        match self {
-            Self::Edit => Some(Action::Edit),
-            Self::Reset => Some(Action::Reset),
-        }
-    }
-}
-
 /// Private to hide enum variants
 #[derive(Debug)]
 enum State {
@@ -249,48 +229,27 @@ impl State {
 
     /// Open a modal to let the user edit temporary override values
     fn open_edit_modal(&self, emitter: Emitter<SaveAuthenticationOverride>) {
-        let (label, value) = match &self {
+        let (label, template) = match &self {
             Self::Basic {
                 username,
                 password,
                 selected_field,
                 ..
             } => match selected_field.data().selected() {
-                BasicFields::Username => {
-                    ("username", username.template().display())
-                }
-                BasicFields::Password => {
-                    ("password", password.template().display())
-                }
+                BasicFields::Username => ("username", username),
+                BasicFields::Password => ("password", password),
             },
-            Self::Bearer { token, .. } => {
-                ("bearer token", token.template().display())
-            }
+            Self::Bearer { token, .. } => ("bearer token", token),
         };
-        TextBoxModal::new(
-            format!("Edit {label}"),
-            TextBox::default()
-                .default_value(value.into_owned())
-                .validator(|value| value.parse::<Template>().is_ok()),
-            move |value| {
-                // Defer the state update into an event, so it can get &mut
-                emitter.emit(SaveAuthenticationOverride(value));
-            },
-        )
-        .open();
+        template.open_edit_modal(format!("Edit {label}"), move |template| {
+            // Defer the state update into an event, so it can get &mut
+            emitter.emit(SaveAuthenticationOverride(template));
+        });
     }
 
     /// Override the value template for whichever field is selected, and
     /// recompute the template preview
-    fn set_override(&mut self, value: &str) {
-        let Some(template) = value
-            .parse::<Template>()
-            // The template *should* always parse because the text box has a
-            // validator, but this is just a safety check
-            .reported(&ViewContext::messages_tx())
-        else {
-            return;
-        };
+    fn set_override(&mut self, template: Template) {
         match self {
             Self::Basic {
                 username,
@@ -361,6 +320,7 @@ mod tests {
     use slumber_util::Factory;
     use terminput::KeyCode;
 
+    /// Test edit basic username+password token via keybinds
     #[rstest]
     fn test_edit_basic(harness: TestHarness, terminal: TestTerminal) {
         let authentication = Authentication::Basic {
@@ -415,6 +375,7 @@ mod tests {
         assert_eq!(component.data().override_value(), None);
     }
 
+    /// Test edit basic username via keybinds
     #[rstest]
     fn test_edit_basic_empty_password(
         harness: TestHarness,
@@ -446,6 +407,7 @@ mod tests {
         );
     }
 
+    /// Test edit bearer token via keybinds
     #[rstest]
     fn test_edit_bearer(harness: TestHarness, terminal: TestTerminal) {
         let authentication = Authentication::Bearer {
@@ -479,7 +441,7 @@ mod tests {
         assert_eq!(component.data().override_value(), None);
     }
 
-    /// Test edit menu action
+    /// Test edit/reset via menu action
     #[rstest]
     fn test_edit_action(harness: TestHarness, terminal: TestTerminal) {
         let authentication = Authentication::Bearer {
@@ -502,6 +464,12 @@ mod tests {
                 token: "i am a token!".into()
             })
         );
+
+        component
+            .int()
+            .action("Reset Authentication")
+            .assert_empty();
+        assert_eq!(component.data().override_value(), None);
     }
 
     /// Basic auth fields should load persisted overrides
@@ -535,7 +503,7 @@ mod tests {
         );
     }
 
-    /// Basic auth fields should load persisted overrides
+    /// Bearer auth fields should load persisted overrides
     #[rstest]
     fn test_persisted_load_bearer(
         harness: TestHarness,
