@@ -15,10 +15,10 @@ pub mod yaml;
 #[cfg(any(test, feature = "test"))]
 pub use test_util::*;
 
-use anyhow::anyhow;
 use itertools::Itertools;
 use serde::{Deserialize, de::Error as _};
 use std::{
+    error::Error,
     fmt::{self, Debug, Display},
     ops::Deref,
     str::FromStr,
@@ -26,7 +26,9 @@ use std::{
 };
 use tracing::error;
 use winnow::{
-    ModalResult, Parser, ascii::digit1, combinator::repeat, token::take_while,
+    ModalResult, Parser,
+    ascii::digit1,
+    combinator::{alt, repeat},
 };
 
 /// Link to the GitHub New Issue form
@@ -72,6 +74,7 @@ impl<'a, T: Copy> Mapping<'a, T> {
             .flat_map(|(_, strings)| strings.iter().copied())
     }
 }
+
 /// Extension trait for [Result]
 pub trait ResultTraced<T, E>: Sized {
     /// If this is an error, trace it. Return the same result.
@@ -79,7 +82,28 @@ pub trait ResultTraced<T, E>: Sized {
     fn traced(self) -> Self;
 }
 
-impl<T> ResultTraced<T, anyhow::Error> for anyhow::Result<T> {
+impl<T, E: 'static + Error> ResultTraced<T, E> for Result<T, E> {
+    fn traced(self) -> Self {
+        self.inspect_err(|err| error!(error = err as &dyn Error))
+    }
+}
+
+/// [ResultTraced] but for the `anyhow` result. This has to be a separate trait
+/// because we can't put a blanket impl on std `Error` *and* `anyhow::Result`,
+/// as the two "could" conflict in the future.
+pub trait ResultTracedAnyhow<T, E>: Sized {
+    /// If this is an error, trace it. Return the same result.
+    #[must_use]
+    fn traced(self) -> Self;
+}
+
+// A blanket impl that covers `anyhow::Error` without actually referring to it.
+// This allows us to omit anyhow as a dependency, so downstream consumers don't
+// pull it in unless they need it.
+impl<T, E> ResultTracedAnyhow<T, E> for Result<T, E>
+where
+    E: Deref<Target = dyn Error + Send + Sync>,
+{
     fn traced(self) -> Self {
         self.inspect_err(|err| error!(error = err.deref()))
     }
@@ -154,7 +178,7 @@ impl Display for TimeSpan {
 }
 
 impl FromStr for TimeSpan {
-    type Err = String;
+    type Err = TimeSpanParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         fn quantity(input: &mut &str) -> ModalResult<u64> {
@@ -162,9 +186,13 @@ impl FromStr for TimeSpan {
         }
 
         fn unit(input: &mut &str) -> ModalResult<DurationUnit> {
-            take_while(1.., char::is_alphabetic)
-                .parse_to()
-                .parse_next(input)
+            alt((
+                "s".map(|_| DurationUnit::Second),
+                "m".map(|_| DurationUnit::Minute),
+                "h".map(|_| DurationUnit::Hour),
+                "d".map(|_| DurationUnit::Day),
+            ))
+            .parse_next(input)
         }
 
         // Parse one or more quantity-unit pairs and sum them all up
@@ -174,17 +202,7 @@ impl FromStr for TimeSpan {
                 |acc, (quantity, unit)| acc + (quantity * unit.seconds()),
             )
             .parse(s)
-            // The format is so simple there isn't much value in spitting out a
-            // specific parsing error, just use a canned one
-            .map_err(|_| {
-                format!(
-                    "Invalid duration, must be `(<quantity><unit>)+` \
-                    (e.g. `12d` or `1h30m`). Units are {}",
-                    DurationUnit::ALL.iter().format_with(", ", |unit, f| f(
-                        &format_args!("`{unit}`")
-                    ))
-                )
-            })?;
+            .map_err(|_| TimeSpanParseError)?;
 
         Ok(Self(Duration::from_secs(seconds)))
     }
@@ -200,6 +218,27 @@ impl<'de> Deserialize<'de> for TimeSpan {
     }
 }
 
+/// Error for [TimeSpan]'s `FromStr` impl
+#[derive(Debug)]
+pub struct TimeSpanParseError;
+
+impl Display for TimeSpanParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        // The format is so simple there isn't much value in spitting out a
+        // specific parsing error, just use a canned one
+        write!(
+            f,
+            "Invalid duration, must be `(<quantity><unit>)+` \
+                (e.g. `12d` or `1h30m`). Units are {}",
+            DurationUnit::ALL
+                .iter()
+                .format_with(", ", |unit, f| f(&format_args!("`{unit}`")))
+        )
+    }
+}
+
+impl Error for TimeSpanParseError {}
+
 /// Supported units for duration parsing/formatting
 #[derive(Debug)]
 enum DurationUnit {
@@ -233,20 +272,6 @@ impl Display for DurationUnit {
     }
 }
 
-impl FromStr for DurationUnit {
-    type Err = anyhow::Error;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_lowercase().as_str() {
-            "s" => Ok(Self::Second),
-            "m" => Ok(Self::Minute),
-            "h" => Ok(Self::Hour),
-            "d" => Ok(Self::Day),
-            _ => Err(anyhow!("Invalid duration unit `{s}`")),
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -310,10 +335,6 @@ mod tests {
         #[case] s: &'static str,
         #[case] expected_error: &str,
     ) {
-        assert_err!(
-            // Map to anyhow error because assert_err! requires it
-            s.parse::<TimeSpan>().map_err(anyhow::Error::msg),
-            expected_error
-        );
+        assert_err(s.parse::<TimeSpan>(), expected_error);
     }
 }
