@@ -7,28 +7,25 @@
 //! by serde/serde_yaml, because there's no need for error messages and the
 //! derive macros are sufficient to generate the corresponding YAML.
 
+mod error;
 mod resolve;
 
+pub use error::{Expected, LocatedError, YamlError, YamlErrorKind};
 #[cfg(feature = "test")]
 pub use test_util::*;
 
-use crate::yaml::resolve::ReferenceError;
 use indexmap::{IndexMap, IndexSet};
-use itertools::Itertools;
 use saphyr::{
     AnnotatedMapping, AnnotatedNode, LoadableYamlNode, MarkedYaml, Marker,
-    Scalar, ScanError, YamlData,
+    Scalar, YamlData,
 };
 use std::{
-    error::Error as StdError,
     fs,
     hash::{Hash, Hasher},
-    io,
     path::{Path, PathBuf},
 };
-use thiserror::Error;
 
-pub type Result<T> = std::result::Result<T, LocatedError<Error>>;
+pub type Result<T> = std::result::Result<T, LocatedError<YamlErrorKind>>;
 
 /// Load YAML from a file and deserialize it into type `T`.
 ///
@@ -37,7 +34,7 @@ pub type Result<T> = std::result::Result<T, LocatedError<Error>>;
 /// data types. We do this rather than use serde_yaml because it provides:
 /// - Better error messages
 /// - Source span tracking
-pub fn deserialize_file<T>(path: &Path) -> anyhow::Result<T>
+pub fn deserialize_file<T>(path: &Path) -> std::result::Result<T, YamlError>
 where
     T: DeserializeYaml,
 {
@@ -48,23 +45,19 @@ where
         .and_then(|yaml| {
             yaml.resolve_references(&mut source_map).map_err(|error| {
                 LocatedError {
-                    error: Error::Reference(error.error),
+                    error: YamlErrorKind::Reference(error.error),
                     location: error.location,
                 }
             })
         })
         // Deserialize as T
-        .and_then(T::deserialize)
-        .map_err(|error| {
-            // Make the location presentable
-            let location = error.location.resolve(&source_map);
-            anyhow::Error::from(error.error)
-                .context(format!("Error at {location}"))
-        })
+        .and_then(|yaml| T::deserialize(yaml, &source_map))
+        // Make the location presentable
+        .map_err(|error| error.resolve(&source_map))
 }
 
 /// Parse a string into YAML, then deserialize it into `T`
-pub fn deserialize_str<T>(yaml: &str) -> anyhow::Result<T>
+pub fn deserialize_str<T>(yaml: &str) -> std::result::Result<T, YamlError>
 where
     T: DeserializeYaml,
 {
@@ -78,7 +71,7 @@ where
 fn deserialize<T>(
     parse_result: Result<SourcedYaml>,
     mut source_map: SourceMap,
-) -> anyhow::Result<T>
+) -> std::result::Result<T, YamlError>
 where
     T: DeserializeYaml,
 {
@@ -87,19 +80,15 @@ where
         .and_then(|yaml| {
             yaml.resolve_references(&mut source_map).map_err(|error| {
                 LocatedError {
-                    error: Error::Reference(error.error),
+                    error: YamlErrorKind::Reference(error.error),
                     location: error.location,
                 }
             })
         })
         // Deserialize as T
-        .and_then(T::deserialize)
-        .map_err(|error| {
-            // Make the location presentable
-            let location = error.location.resolve(&source_map);
-            anyhow::Error::from(error.error)
-                .context(format!("Error at {location}"))
-        })
+        .and_then(|yaml| T::deserialize(yaml, &source_map))
+        // Make the location presentable
+        .map_err(|error| error.resolve(&source_map))
 }
 
 /// Deserialize from YAML into the implementing type
@@ -108,7 +97,7 @@ pub trait DeserializeYaml: Sized {
     fn expected() -> Expected;
 
     /// Deserialize the given YAML value into this type
-    fn deserialize(yaml: SourcedYaml) -> Result<Self>;
+    fn deserialize(yaml: SourcedYaml, source_map: &SourceMap) -> Result<Self>;
 }
 
 /// Implement [DeserializeYaml] for a type `T` via type `U`, where `T: From<U>,
@@ -123,8 +112,10 @@ macro_rules! impl_deserialize_from {
 
             fn deserialize(
                 yaml: slumber_util::yaml::SourcedYaml,
+                source_map: &slumber_util::yaml::SourceMap,
             ) -> slumber_util::yaml::Result<Self> {
-                <$u as DeserializeYaml>::deserialize(yaml).map(<$t>::from)
+                <$u as DeserializeYaml>::deserialize(yaml, source_map)
+                    .map(<$t>::from)
             }
         }
     };
@@ -136,6 +127,8 @@ macro_rules! impl_deserialize_from {
 #[macro_export]
 macro_rules! deserialize_enum {
     ($yaml:expr, $($tag:literal => $f:expr),* $(,)?) => {
+            use $crate::yaml::{LocatedError, YamlErrorKind};
+
             const TYPE_FIELD: &str = "type";
             const EXPECTED: Expected =
                 Expected::OneOf(&[$(&Expected::Literal($tag),)*]);
@@ -146,7 +139,7 @@ macro_rules! deserialize_enum {
             let kind_yaml = mapping
                 .remove(&SourcedYaml::value_from_str(TYPE_FIELD))
                 .ok_or(LocatedError {
-                    error: Error::MissingField {
+                    error: YamlErrorKind::MissingField {
                         field: TYPE_FIELD,
                         expected: EXPECTED,
                     },
@@ -164,7 +157,7 @@ macro_rules! deserialize_enum {
                 $($tag => $f(yaml),)*
                 // Unknown tag
                 _ => Err(LocatedError {
-                    error: Error::Unexpected {
+                    error: YamlErrorKind::Unexpected {
                         expected: EXPECTED,
                         actual: format!("{kind:?}"),
                     },
@@ -179,7 +172,7 @@ impl DeserializeYaml for bool {
         Expected::Boolean
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, _source_map: &SourceMap) -> Result<Self> {
         yaml.try_into_bool()
     }
 }
@@ -189,7 +182,7 @@ impl DeserializeYaml for usize {
         Expected::Number
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, _source_map: &SourceMap) -> Result<Self> {
         yaml.try_into_usize()
     }
 }
@@ -199,7 +192,7 @@ impl DeserializeYaml for String {
         Expected::String
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, _source_map: &SourceMap) -> Result<Self> {
         yaml.try_into_string()
     }
 }
@@ -213,11 +206,11 @@ impl<T: DeserializeYaml> DeserializeYaml for Option<T> {
         T::expected()
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, source_map: &SourceMap) -> Result<Self> {
         if yaml.data.is_null() {
             Ok(None)
         } else {
-            T::deserialize(yaml).map(Some)
+            T::deserialize(yaml, source_map).map(Some)
         }
     }
 }
@@ -230,9 +223,12 @@ where
         Expected::Sequence
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, source_map: &SourceMap) -> Result<Self> {
         let sequence = yaml.try_into_sequence()?;
-        sequence.into_iter().map(T::deserialize).collect()
+        sequence
+            .into_iter()
+            .map(|yaml| T::deserialize(yaml, source_map))
+            .collect()
     }
 }
 
@@ -246,10 +242,15 @@ where
         Expected::Mapping
     }
 
-    fn deserialize(yaml: SourcedYaml) -> Result<Self> {
+    fn deserialize(yaml: SourcedYaml, source_map: &SourceMap) -> Result<Self> {
         yaml.try_into_mapping()?
             .into_iter()
-            .map(|(k, v)| Ok((K::deserialize(k)?, V::deserialize(v)?)))
+            .map(|(k, v)| {
+                Ok((
+                    K::deserialize(k, source_map)?,
+                    V::deserialize(v, source_map)?,
+                ))
+            })
             .collect()
     }
 }
@@ -262,7 +263,7 @@ where
 /// displayed.
 #[derive(Clone, Debug, Eq)]
 pub struct SourcedYaml<'input> {
-    pub location: SourceLocation,
+    pub location: SourceIdLocation,
     pub data: YamlData<'input, Self>,
 }
 
@@ -271,11 +272,11 @@ impl<'input> SourcedYaml<'input> {
     fn load(path: &Path, source: SourceId) -> Result<Self> {
         let content =
             fs::read_to_string(path).map_err(|error| LocatedError {
-                error: Error::Io {
+                error: YamlErrorKind::Io {
                     error,
                     source: path.display().to_string(),
                 },
-                location: SourceLocation::default(),
+                location: SourceIdLocation::default(),
             })?;
         Self::load_from_str(&content, source)
     }
@@ -300,7 +301,8 @@ impl<'input> SourcedYaml<'input> {
     /// Convert a [MarkedYaml] to [SourcedYaml] by transforming `Marker` spans
     /// to [SourceLocation]
     fn from_marked_yaml(yaml: MarkedYaml<'input>, source_id: SourceId) -> Self {
-        let location = SourceLocation::from_marker(source_id, yaml.span.start);
+        let location =
+            SourceIdLocation::from_marker(source_id, yaml.span.start);
 
         let data = match yaml.data {
             YamlData::Value(scalar) => YamlData::Value(scalar),
@@ -379,7 +381,7 @@ impl<'input> SourcedYaml<'input> {
             // provide a helpful error
             if mapping.contains_key(&SourcedYaml::value_from_str("<<")) {
                 Err(LocatedError {
-                    error: Error::UnsupportedMerge,
+                    error: YamlErrorKind::UnsupportedMerge,
                     location: self.location,
                 })
             } else {
@@ -394,7 +396,7 @@ impl<'input> SourcedYaml<'input> {
     pub fn value_from_str(value: &'input str) -> Self {
         Self {
             data: YamlData::Value(Scalar::parse_from_cow(value.into())),
-            location: SourceLocation::default(),
+            location: SourceIdLocation::default(),
         }
     }
 
@@ -402,7 +404,7 @@ impl<'input> SourcedYaml<'input> {
     fn value_from_string(value: String) -> Self {
         Self {
             data: YamlData::Value(Scalar::parse_from_cow(value.into())),
-            location: SourceLocation::default(),
+            location: SourceIdLocation::default(),
         }
     }
 
@@ -421,7 +423,7 @@ impl<'a> From<YamlData<'a, SourcedYaml<'a>>> for SourcedYaml<'a> {
     fn from(value: YamlData<'a, SourcedYaml<'a>>) -> Self {
         Self {
             data: value,
-            location: SourceLocation::default(),
+            location: SourceIdLocation::default(),
         }
     }
 }
@@ -458,7 +460,7 @@ impl AnnotatedNode for SourcedYaml<'_> {
 /// clone expensive paths into each one. The map also makes it easy to cache
 /// sources that get referenced multiple times.
 #[derive(Debug, Default)]
-struct SourceMap {
+pub struct SourceMap {
     /// A bijection between source IDs and paths. Each path is unique and gets
     /// assigned a unique ID. The ID is an index into this set.
     sources: IndexSet<PathBuf>,
@@ -523,9 +525,11 @@ enum SourceId {
     Memory,
 }
 
-/// Source location of a YAML value
+/// Source location of a YAML value, where the source is a cheap ID.
+///
+/// To resolve this into a [SourceLocation], use [Self::resolve].
 #[derive(Copy, Clone, Debug, Default, Eq, Hash, PartialEq)]
-pub struct SourceLocation {
+pub struct SourceIdLocation {
     /// Source of the YAML document
     source: SourceId,
     /// 1-indexed line in the document
@@ -534,7 +538,7 @@ pub struct SourceLocation {
     column: u32,
 }
 
-impl SourceLocation {
+impl SourceIdLocation {
     fn from_marker(source_id: SourceId, marker: Marker) -> Self {
         Self {
             source: source_id,
@@ -547,12 +551,12 @@ impl SourceLocation {
     /// Resolve this source location by mapping its source ID to the
     /// corresponding string. This makes the location ready for display, at
     /// the cost of making it no longer `Copy`.
-    fn resolve(&self, source_map: &SourceMap) -> ResolvedSourceLocation {
+    pub fn resolve(&self, source_map: &SourceMap) -> SourceLocation {
         let source = source_map
             .get_path(self.source)
             .map(|path| path.display().to_string())
             .unwrap_or_default();
-        ResolvedSourceLocation {
+        SourceLocation {
             source,
             line: self.line,
             column: self.column,
@@ -560,168 +564,28 @@ impl SourceLocation {
     }
 }
 
-/// [SourceLocation], but the source name has been resolved from the source map
-/// to be presentable to the user
-#[derive(Clone, Debug, Default, derive_more::Display, Eq, Hash, PartialEq)]
-#[display("{source}:{line}:{column}")]
-pub struct ResolvedSourceLocation {
-    /// Display name (e.g. path) for the YAML source
-    source: String,
-    /// 1-indexed line in the file
-    line: u32,
-    /// 1-indexed column in the file
-    column: u32,
-}
-
-/// An error paired with the source location in YAML where the error occurred
+/// Location in a YAML document.
 ///
-/// This doesn't implement `Error` because this isn't immediately displayable.
-/// The location needs to be resolved to make this presentable.
-#[derive(Debug)]
-pub struct LocatedError<E> {
-    /// Error that occurred
-    pub error: E,
-    /// Source location of the error. This is an *unresolved* location, meaning
-    /// it contains a source ID instead of a source path.
-    pub location: SourceLocation,
+/// The source has been rendered to a presentable string (probably a path).
+#[derive(Clone, Debug, Default, derive_more::Display)]
+#[display("{source}:{line}:{column}")]
+pub struct SourceLocation {
+    /// Display name (e.g. path) for the YAML source
+    pub source: String,
+    /// 1-indexed line in the file
+    pub line: u32,
+    /// 1-indexed column in the file
+    pub column: u32,
 }
 
-impl<E> LocatedError<E> {
-    /// Move the inner error out
-    pub fn into_error(self) -> E {
-        self.error
+// Ignore location in tests for simpler assertions
+#[cfg(feature = "test")]
+impl PartialEq for SourceLocation {
+    fn eq(&self, _: &Self) -> bool {
+        true
     }
 }
 
-impl LocatedError<Error> {
-    /// Create a new [Other](Self::Other) from any error type
-    pub fn other(
-        error: impl Into<Box<dyn StdError + Send + Sync>>,
-        location: SourceLocation,
-    ) -> Self {
-        Self {
-            error: Error::Other(error.into()),
-            location,
-        }
-    }
-
-    fn scan(error: ScanError, source_id: SourceId) -> Self {
-        let location = SourceLocation::from_marker(source_id, *error.marker());
-        Self {
-            error: Error::Scan(error),
-            location,
-        }
-    }
-
-    /// Create a new [UnexpectedType](Self::UnexpectedType) from the expected
-    /// type and actual value
-    pub fn unexpected(expected: Expected, actual: SourcedYaml) -> Self {
-        // Find a useful representation of the received value
-        let actual_string = match actual.data {
-            // Scalars are unlikely to be big so we can include the actual value
-            YamlData::Value(Scalar::Null) => "null".into(),
-            YamlData::Value(Scalar::Boolean(b)) => format!("`{b}`"),
-            YamlData::Value(Scalar::Integer(i)) => format!("`{i}`"),
-            YamlData::Value(Scalar::FloatingPoint(f)) => format!("`{f}`"),
-            // Use debug format to get wrapping quotes
-            YamlData::Value(Scalar::String(s)) => format!("{s:?}"),
-            YamlData::Tagged(tag, _) => format!("tag `{tag}`"),
-            // Collections could be large so just include the type
-            YamlData::Sequence(_) => "sequence".into(),
-            YamlData::Mapping(_) => "mapping".into(),
-            YamlData::Representation(_, _, _)
-            | YamlData::Alias(_)
-            | YamlData::BadValue => yaml_parse_panic(),
-        };
-        Self {
-            location: actual.location,
-            error: Error::Unexpected {
-                expected,
-                actual: actual_string,
-            },
-        }
-    }
-}
-
-/// An error that can occur while deserializing a YAML value
-#[derive(Debug, Error)]
-pub enum Error {
-    #[error("Error opening {source}")]
-    Io {
-        #[source]
-        error: io::Error,
-        source: String,
-    },
-
-    #[error("Expected field `{field}` with {expected}")]
-    MissingField {
-        field: &'static str,
-        expected: Expected,
-    },
-
-    /// External error type
-    #[error(transparent)]
-    Other(Box<dyn 'static + StdError + Send + Sync>),
-
-    /// Error parsing or resolving a reference under a `$ref` tag
-    #[error(transparent)]
-    Reference(ReferenceError),
-
-    /// Error parsing YAML
-    #[error(transparent)]
-    Scan(saphyr::ScanError),
-
-    /// Expected a particular type or value, but received something else
-    #[error("Expected {expected}, received {actual}")]
-    Unexpected {
-        expected: Expected,
-        /// Pre-formatted "actual" value. Getting an owned YAML value from
-        /// is complicated so it's easier to store it as the presentation
-        /// string
-        actual: String,
-    },
-
-    /// Struct received an extra field
-    #[error("Unexpected field `{0}`")]
-    UnexpectedField(String),
-
-    /// Special error case to identify the `<<` key. We want to report this in
-    /// both static and dynamic mappings because the user almost definitely
-    /// doesn't want the literal key `<<`.
-    #[error("YAML merge syntax `<<` is not supported")]
-    UnsupportedMerge,
-}
-
-/// When a value is expected but is either incorrect or missing, this type
-/// allows the caller to declare what they expected to find
-#[derive(Debug, derive_more::Display)]
-pub enum Expected {
-    /// Expected null
-    #[display("null")]
-    Null,
-    /// Expected a string
-    #[display("string")]
-    String,
-    /// Expected a boolean
-    #[display("boolean")]
-    Boolean,
-    /// Expected an integer or float
-    #[display("number")]
-    Number,
-    /// Expected a sequence
-    #[display("sequence")]
-    Sequence,
-    /// Expected a mapping
-    #[display("mapping")]
-    Mapping,
-    /// Expected a string literal
-    #[display("{_0:?}")]
-    Literal(&'static str),
-    /// Expected one of a static set of types (for enum discriminants)
-    #[display("one of {}", _0.iter().format(", "))]
-    OneOf(&'static [&'static Self]),
-}
-
 /// Utility for deserializing a struct or enum variant from a YAML mapping.
 /// Initialize this struct with a YAML value, and it will:
 /// - Ensure the value is a mapping
@@ -730,7 +594,7 @@ pub enum Expected {
 ///     - NOTE: `done` needs to be called manually after deserialization!
 pub struct StructDeserializer<'a> {
     pub mapping: AnnotatedMapping<'a, SourcedYaml<'a>>,
-    pub location: SourceLocation,
+    pub location: SourceIdLocation,
 }
 
 impl<'a> StructDeserializer<'a> {
@@ -741,17 +605,21 @@ impl<'a> StructDeserializer<'a> {
     }
 
     /// Deserialize a field from the mapping
-    pub fn get<T: DeserializeYaml>(&mut self, field: Field<T>) -> Result<T> {
+    pub fn get<T: DeserializeYaml>(
+        &mut self,
+        field: Field<T>,
+        source_map: &SourceMap,
+    ) -> Result<T> {
         if let Some(value) = self
             .mapping
             .remove(&SourcedYaml::value_from_str(field.name))
         {
-            T::deserialize(value)
+            T::deserialize(value, source_map)
         } else if let Some(default) = field.default {
             Ok(default)
         } else {
             Err(LocatedError {
-                error: Error::MissingField {
+                error: YamlErrorKind::MissingField {
                     field: field.name,
                     expected: T::expected(),
                 },
@@ -767,7 +635,7 @@ impl<'a> StructDeserializer<'a> {
             // If the key isn't a string, it's reasonable to return a type error
             let key = key.try_into_string()?;
             Err(LocatedError {
-                error: Error::UnexpectedField(key),
+                error: YamlErrorKind::UnexpectedField(key),
                 location: key_location,
             })
         } else {
@@ -827,7 +695,7 @@ pub fn yaml_parse_panic() -> ! {
 #[cfg(feature = "test")]
 mod test_util {
     use super::{DeserializeYaml, Result, SourcedYaml};
-    use crate::yaml::SourceId;
+    use crate::yaml::{SourceId, SourceMap};
     use std::iter;
 
     /// Deserialize a [serde_yaml::Value] using saphyr. Serde values are easier
@@ -837,7 +705,8 @@ mod test_util {
     ) -> Result<T> {
         let yaml_input = serde_yaml::to_string(&yaml).unwrap();
         let yaml = SourcedYaml::load_from_str(&yaml_input, SourceId::Memory)?;
-        T::deserialize(yaml)
+        let source_map = SourceMap::default();
+        T::deserialize(yaml, &source_map)
     }
 
     /// Build a YAML mapping
